--[[

    File: util.lua 

    Copyright (C) 2000-2009 Christopher Moore (christopher.e.moore@gmail.com)
	  
    This software is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
  
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
  
    You should have received a copy of the GNU General Public License along
    with this program; if not, write the Free Software Foundation, Inc., 51
    Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

--]]

do
	local f = io.open('luaext', 'r')
	if not f then
		package.path = package.path .. ';../?.lua'
	end
end

require 'luaext.table'
require 'luaext.string'
require 'luaext.io'
require 'luaext.math'


-- misc

-- TODO - replace all .new's with :new's, and then give all base classes this member method, and from there don't worry about recreating each class's new method
function new(class, ...)
	return setmetatable({}, class):init(...)
end

-- until then...
function newmember(class)
	return function(...)
		local obj = setmetatable({class=class}, class)
		if obj.init then obj:init(...) end
		return obj
	end
end

-- and not to undermine the needed change...
function class(...)
	local cl = table.new(...)	-- TODO - this is a no-no.  don't clone table's members as well.
	local parents = {...}

	-- part of me wants to make it always a table
	-- but I know that'll be utilized 10% of the time
	if #parents == 0 then
		cl.super = nil
	elseif #parents == 1 then
		cl.super = parents[1]
	else
		cl.super = parents
	end
	
	cl.__index = cl
	cl.new = newmember(cl)
	
	setmetatable(cl, {__call = function(self, ...)
		return self.new(...)
	end;})
	
	return cl
end

